@@ -404,12 +404,9 @@ create_ps = function(fn) {
 #' @param shapes_out (`function` or `NULL` or `"infer"`)\cr
 #'   The private `.shapes_out(shapes_in, param_vals, task)` method of [`PipeOpTaskPreprocTorch`]
 #'   (see section Inheriting).
-#'   Special values are `NULL` and `infer`:
+#'   Special values are `NULL` and `"infer"`:
 #'   If `NULL`, the output shapes are unknown.
-#'   If "infer", the output shape function is inferred and calculates the output shapes as follows:
-#'   For an input shape of (NA, ...) a meta-tensor of shape (1, ...) is created and the preprocessing function is
-#'   applied. Afterwards the batch dimension (1) is replaced with NA and the shape is returned.
-#'   If the first dimension is not `NA`, the output shape of applying the preprocessing function is returned.
+#'   Option `"infer"` uses [`infer_shapes`].
 #'   Method `"infer"` should be correct in most cases, but might fail in some edge cases.
 #' @param param_set ([`ParamSet`][paradox::ParamSet] or `NULL`)\cr
 #'   The parameter set.
@@ -452,31 +449,8 @@ pipeop_preproc_torch = function(id, fn, shapes_out = NULL, param_set = NULL, pac
   # we e.g. want torchvision in suggests, so we cannot already access the function.
   if (identical(shapes_out, "infer")) {
     shapes_out = crate(function(shapes_in, param_vals, task) {
-      sin = shapes_in[[1L]]
-      batch_dim = sin[1L]
-      batchdim_is_unknown = is.na(batch_dim)
-      if (batchdim_is_unknown) {
-        sin[1] = 1L
-      }
-      if (self$rowwise) {
-        sin = sin[-1L]
-      }
-      tensor_in = mlr3misc::invoke(torch_empty, .args = sin, device = torch_device("meta"))
-      tensor_out = tryCatch(mlr3misc::invoke(self$fn, tensor_in, .args = param_vals),
-        error = function(e) {
-          stopf("Input shape '%s' is invalid for PipeOp with id '%s'.", shape_to_str(list(sin)), self$id)
-        }
-      )
-      sout = dim(tensor_out)
-
-      if (self$rowwise) {
-        sout = c(batch_dim, sout)
-      } else if (batchdim_is_unknown) {
-        sout[1] = NA
-      }
-
-      list(sout)
-    }, .parent = topenv())
+      getFromNamespace("infer_shapes", "mlr3torch")(shapes_in = shapes_in, param_vals = param_vals, output_names = self$output$name, fn = self$fn, rowwise = self$rowwise, id = self$id) # nolint
+    })
   } else if (is.function(shapes_out) || is.null(shapes_out)) {
     # nothing to do
   } else {
