@@ -35,6 +35,9 @@ PipeOpTorchConvTranspose = R6Class("PipeOpTorchConvTranspose",
     }
   ),
   private = list(
+    .additional_phash_input = function() {
+      list(private$.d)
+    },
     .shapes_out = function(shapes_in, param_vals, task) {
       list(conv_transpose_output_shape(
         shape_in = shapes_in[[1]],
