@@ -107,14 +107,6 @@ load_col_info = function(name) {
   readRDS(system.file("col_info", paste0(name, ".rds"), package = "mlr3torch"))
 }
 
-get_nout = function(task) {
-  switch(task$task_type,
-    regr = 1,
-    classif = length(task$class_names),
-    stopf("Unknown task type '%s'.", task$task_type)
-  )
-}
-
 
 test_equal_col_info = function(x, y) {
   nms = c("id", "type", "levels")
@@ -155,6 +147,14 @@ uniqueify = function(new, existing) {
 }
 
 shape_to_str = function(x) {
+  assert(test_list(x) || test_integerish(x) || is.null(x))
+  if (test_integerish(x)) { # single shape
+    return(sprintf("(%s)", paste0(x, collapse = ",")))
+  }
+  if (is.null(x)) {
+    return("(<unknown>)")
+  }
+
   shapedescs = map_chr(x, function(y) {
     if (is.null(y)) {
       return("<unknown>")
@@ -190,7 +190,10 @@ list_to_batch = function(tensors) {
 }
 
 auto_cache_lazy_tensors = function(lts) {
-  any(duplicated(map_chr(lts, function(x) dd(x)$dataset_hash)))
+  if (length(lts) <= 1L) {
+    return(FALSE)
+  }
+  anyDuplicated(unlist(map_if(lts, function(x) length(x) > 0, function(x) dd(x)$dataset_hash))) > 0L
 }
 
 #' Replace the head of a network
@@ -209,6 +212,10 @@ check_nn_module = function(x) {
   check_class(x, "nn_module")
 }
 
+check_nn_module_generator = function(x) {
+  check_class(x, "nn_module_generator")
+}
+
 check_callbacks = function(x) {
   if (test_class(x, "TorchCallback")) {
     x = list(x)
@@ -267,3 +274,55 @@ order_named_args = function(f, l) {
   }
   l2
 }
+
+
+#' @title Network Output Dimension
+#' @description
+#' Calculates the output dimension of a neural network for a given task that is expected by
+#' \pkg{mlr3torch}.
+#' For classification, this is the number of classes (unless it is a binary classification task,
+#' where it is 1). For regression, it is 1.
+#' @param x (any)\cr
+#'   The task.
+#' @param ... (any)\cr
+#'   Additional arguments. Not used yet.
+#' @export
+output_dim_for = function(x, ...) {
+  UseMethod("output_dim_for")
+}
+
+#' @export
+output_dim_for.TaskClassif = function(x, ...) {
+  if ("twoclass" %in% x$properties) {
+    return(1L)
+  }
+  length(x$class_names)
+}
+
+#' @export
+output_dim_for.TaskRegr = function(x, ...) {
+  1L
+}
+
+all_or_none_ = function(...) {
+  args = list(...)
+  all_none = all(sapply(args, is.null))
+  all_not_none = all(!sapply(args, is.null))
+  return(all_none || all_not_none)
+}
+
+single_lazy_tensor = function(task) {
+  identical(task$feature_types[, "type"][[1L]], "lazy_tensor")
+}
+                              
+n_num_features = function(task) {
+  sum(task$feature_types$type %in% c("numeric", "integer"))
+}
+
+n_categ_features = function(task) {
+  sum(task$feature_types$type %in% c("factor", "ordered", "logical"))
+}
+
+n_ltnsr_features = function(task) {
+  sum(task$feature_types$type == "lazy_tensor")
+}
\ No newline at end of file
