@@ -28,6 +28,10 @@ expect_pipeop_torch = function(graph, id, task, module_class = id, exclude_args
   result = graph$train(task)
   md = result[[1]]
 
+  # PipeOp overwrites hash and phash if necessary
+  testthat::expect_warning(po_test$hash, regexp = NA)
+  testthat::expect_warning(po_test$phash, regexp = NA)
+
   modulegraph = md$graph
   po_module = modulegraph$pipeops[[id]]
   if (is.null(po_module$module)) {
@@ -76,7 +80,6 @@ expect_pipeop_torch = function(graph, id, task, module_class = id, exclude_args
   # We need to match the names out the output A1 to the names of A and then get the input channels of B
   # (which is the pipeop we are testing)
 
-
   # input pointer (for po_test)
   ip = pmap(graph$edges[get("dst_id") == id, list(x = get("src_id"), y = get("src_channel"))], function(x, y) c(x, y))
 
@@ -85,7 +88,6 @@ expect_pipeop_torch = function(graph, id, task, module_class = id, exclude_args
   pointers = append(ip, op)
   net = model_descriptor_to_module(md, output_pointers = pointers, list_output = TRUE)$to(device = "cpu")
 
-
   ds = task_dataset(task, md$ingress)
   batch = ds$.getbatch(1)
   out = with_no_grad(invoke(net$forward, .args = batch$x))
@@ -305,10 +307,40 @@ expect_torch_callback = function(torch_callback, check_man = TRUE, check_paramse
     paramtest = expect_paramset(torch_callback$param_set, init_fn)
     expect_paramtest(paramtest)
   }
-  implemented_stages = names(cbgen$public_methods)[grepl("^on_", names(cbgen$public_methods))]
+
+  get_all_implemented_stages = function(cbgen) {
+    implemented_stages = character(0)
+    current_gen = cbgen
+
+    while (!is.null(current_gen)) {
+      current_stages = names(current_gen$public_methods)[grepl("^on_", names(current_gen$public_methods))]
+      implemented_stages = union(implemented_stages, current_stages)
+
+      current_gen = current_gen$get_inherit()
+    }
+
+    return(implemented_stages)
+  }
+  
+  implemented_stages = get_all_implemented_stages(cbgen)
   expect_subset(implemented_stages, mlr_reflections$torch$callback_stages)
   expect_true(length(implemented_stages) > 0)
-  walk(implemented_stages, function(stage) expect_function(cbgen$public_methods[[stage]], nargs = 0))
+
+  check_stage_function = function(stage, cbgen) {
+    current_gen = cbgen
+
+    while (!is.null(current_gen)) {
+      if (stage %in% names(current_gen$public_methods)) {
+        expect_function(current_gen$public_methods[[stage]], nargs = 0)
+        return()
+      }
+      current_gen = current_gen$get_inherit()
+    }
+  }
+
+  walk(implemented_stages, function(stage) {
+    check_stage_function(stage, cbgen)
+  })
 
   cb = torch_callback$generate()
   expect_deep_clone(cb, cb$clone(deep = TRUE))
@@ -325,6 +357,10 @@ expect_pipeop_torch_preprocess = function(obj, shapes_in, exclude = character(0)
   if (is.null(seed)) {
     seed = sample.int(100000, 1)
   }
+  # overwrites .additional_phash_input where necessary
+  testthat::expect_warning(obj$hash, regexp = NA)
+  testthat::expect_warning(obj$phash, regexp = NA)
+
   expect_pipeop(obj)
   expect_class(obj, "PipeOpTaskPreprocTorch")
   # a) Check that all parameters but stages have tags train and predict (this should hold in basically all cases)
@@ -423,12 +459,17 @@ expect_pipeop_torch_preprocess = function(obj, shapes_in, exclude = character(0)
 
 expect_learner_torch = function(learner, task, check_man = TRUE, check_id = TRUE) {
   checkmate::expect_class(learner, "LearnerTorch")
+
+  # overwrites .additional_phash_input where necessary
+  testthat::expect_warning(learner$hash, regexp = NA)
+  testthat::expect_warning(learner$phash, regexp = NA)
+
   get("expect_learner", envir = .GlobalEnv)(learner)
   # state cloning is tested separately
   learner1 = learner
   learner1$state = NULL
   expect_deep_clone(learner1, learner1$clone(deep = TRUE))
-  rr = resample(task, learner, rsmp("holdout"))
+  rr = resample(task, learner, rsmp("holdout"), store_models = TRUE)
   expect_double(rr$aggregate())
   checkmate::expect_class(rr, "ResampleResult")
   if (check_id) testthat::expect_true(startsWith(learner$id, learner$task_type))
@@ -436,6 +477,8 @@ expect_learner_torch = function(learner, task, check_man = TRUE, check_id = TRUE
   checkmate::expect_subset(c("mlr3", "mlr3torch", "torch"), learner$packages)
   testthat::expect_true(all(map_lgl(learner$tags, function(tags) "predict" %in% tags || "train" %in% tags)))
 
+  learner = rr$learners[[1L]]
+
   learner$param_set$set_values(device = "cpu")
   ds = learner$dataset(task)
   batch = if (is.null(ds$.getbatch)) {
@@ -447,8 +490,28 @@ expect_learner_torch = function(learner, task, check_man = TRUE, check_id = TRUE
   testthat::expect_true(batch$y$device == torch_device("cpu"))
   if (task$task_type == "regr") {
     testthat::expect_true(batch$y$dtype == torch_float())
-  } else {
+  } else if (task$task_type == "classif" && "twoclass" %in% task$properties) {
+    testthat::expect_true(batch$y$dtype == torch_float())
+  } else if (task$task_type == "classif") {
     testthat::expect_true(batch$y$dtype == torch_long())
+  } else {
+    stopf("Task type %s not yet implemented", task$task_type)
   }
   testthat::expect_true(batch$.index$dtype == torch_long())
+
+  pred = if (length(batch$x) == 1L) {
+    invoke(learner$model$network, .args = unname(batch$x))
+  } else {
+    invoke(learner$model$network, .args = batch$x)
+  }
+
+  if (task$task_type == "classif" && "twoclass" %in% task$properties) {
+    testthat::expect_equal(pred$shape, c(1, 1))
+  } else if (task$task_type == "classif") {
+    testthat::expect_equal(pred$shape, c(1, length(task$class_names)))
+  } else if (task$task_type == "regr") {
+    testthat::expect_equal(pred$shape, c(1, 1))
+  } else {
+    stopf("Task type %s not yet implemented", task$task_type)
+  }
 }
