@@ -7,11 +7,9 @@ old_opts = options(
 # https://github.com/HenrikBengtsson/Wishlist-for-R/issues/88
 old_opts = lapply(old_opts, function(x) if (is.null(x)) FALSE else x)
 
-lg_mlr3 = lgr::get_logger("mlr3")
-old_threshold_mlr3 = lg_mlr3$threshold
-lg_mlr3$set_threshold("warn")
-
+lg = lgr::get_logger("mlr3")
 old_threshold = lg$threshold
-old_plan = future::plan()
 lg$set_threshold("warn")
+
+old_plan = future::plan()
 future::plan("sequential")
